package net.multicast;

import futils.Futil;
import futils.ReaderUtil;

import java.io.*;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;

/**
 * Copyright DocJava, inc. User: lyon Date: Oct 27, 2004 Time: 4:59:20 AM
 */
public class ChatMain {
    public static void main(String[] args) throws IOException {
        int port = 1234;
        InetAddress inetAddress = Utils.getInetAddress();
        try {
            MulticastSocket mcastSocket = Utils.getMulticastSocket(
                    port, inetAddress);

            processWithMulticastSocket(mcastSocket, inetAddress, port);

            Utils.closeSocket(mcastSocket, inetAddress);
        } catch (Exception err) {
            err.printStackTrace();
        }
    }

    public static void processWithMulticastSocket(
            MulticastSocket mcastSocket,
            InetAddress inetAddress,
            int port) throws IOException {
        String strin;
        // ChatThread will handle the incoming
        // Data and print it out to STDN output.
        new ChatThread(mcastSocket);

        // Now read from STDN input and send the Data to the Group.

        BufferedReader br = new BufferedReader(
                new InputStreamReader(System.in));
        System.out.println("Type anything followed by RETURN," +
                " or Ctrl+D to  terminate the program.");
        for (; ;) {
            //Read from the STDN input
            strin = br.readLine();
            if (strin.startsWith("open:")) {
                sendAFile(inetAddress,
                        port,
                        mcastSocket);
                continue;
            }
            if (strin == null) break; // User Hit Ctrl+D
            DatagramPacket dp = new DatagramPacket(strin.getBytes(),
                    strin.length(),
                    inetAddress,
                    port);
            mcastSocket.send(dp);
        }
        System.out.println("Leaving the Group...");
    }

    private static void sendAFile(InetAddress inetAddress,
                                  int port,
                                  MulticastSocket mcastSocket) {
        System.out.println("SendAFile_");
        File f = Futil.getReadFile("select a text file");
        sendAFile(f,
                inetAddress,
                port,
                mcastSocket);
    }

    private static void sendAFile(File f,
                                  InetAddress inetAddress,
                                  int port,
                                  MulticastSocket mcastSocket) {
        try {
            String sa[] = ReaderUtil.getFile(f);
            byte b[] = ReaderUtil.getBytes(sa);
            DatagramPacket dp = new DatagramPacket(b,
                    b.length,
                    inetAddress,
                    port);
            mcastSocket.send(dp);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}

